function [mpP]=GetDiameter_MPScan_L4_cc(fname,matfilename,theframes,window_time_size,the_x,the_decimate,user_microns_per_pixel,channel);
%funtion for loading *.MPD linescan files and gets diameter using fwhm code
%
%becase MPScan gets line scans in both directions, the frame is split into even
%and odd lines and velocity is extravted independently from each one and then averaged
%INPUTS:
%fname-filename
%matfilename-matalab file containg scan path etc
%theframes - beginning and ending frames
%windowtimesize - size (in sec) of the advancemt of the Radon window
%anglespan - how many degrees (+/-) to look in the adaptive search
%angle_resolution - fine angle resolution
%the_x - [left  right] bounds of which to extract from the frames in the X
%dimension
%the_decimate - factor by which to decimate the line scan in the X dimension
%max_v - time points above this absolute speed are clipped out and interpolated
%between the last and next valid time points
%the_objective - a number designating which objective is used

%OUTPUTS
%diamter-dimeter of vessel in microns
%mpP -  a structure with the relevant data
%mpP.Header substruct
%mpP.Ch1 first two frames from channel 1
%mpP.Diameter structure with velocity info etc.
%Open stream

figure(1)
close(1)
f1 = figure(1);
set(f1,'visible','on');
mpfile = actxcontrol('MPfile.Data',[0,0,500,500],f1);
OpenFile = invoke(mpfile,'OpenMPFile',fname);

%Read header and take further action based on header information
mpP.Header = mp2mat_readHeader(mpfile,fname);
mpP.num_frames = theframes(end)-theframes(1);%str2double(mpContent.Header.Frame_Count);
mpP.xsize = str2double(mpP.Header.Frame_Width);
mpP.ysize = str2double(mpP.Header.Frame_Height);
mpP.xstart=the_x(1);
mpP.xstop=the_x(end);
frame_height=mpP.ysize;
for chNum = channel%1 : 4
    chNumStr = num2str(chNum);
    if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
        mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[1 2],the_x);
    end
end


if length(matfilename)>0
    load(matfilename)
    mpP.scanData=scanData;
else
    mpP.scan_path=[];
end

first_frames=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, theframes(1):theframes(1)+10,the_x);%get first 10 frames for background subtraction
time_per_line=str2num(mpP.Header.Frame_Width)*(5/4)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
if strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
    time_per_line=str2num(mpP.Header.Frame_Width)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
end
mpP.Header.LineRate=1/time_per_line;
Fs_Diameter=mpP.Header.LineRate;
windowsizea=round(Fs_Diameter*window_time_size);%*.25 %for interlevaed lines
%adapt windowsize to
windowsize=4*round(windowsizea/4);

mpP.Diameter.windowsize=windowsize;
%%%%basic stuff
nframes=theframes(2)-theframes(1)+1;
stepsize=.25*windowsize;
nlines=nframes*str2num(mpP.Header.Frame_Height);
npoints=max(the_x)-min(the_x)+1;%str2num(mpP.Header.Frame_Width);

nsteps=floor(nlines/stepsize)-3;
% angles=(0:179);%parameters for initial search
% angles_baseline=0:15:179;
% angles_adaptive=[-angle_span:1:angle_span];
% angles_fine=-1.75:angle_resolution:1.75;%fine grained search
% n_adaptive_angles=length(angles_adaptive);
% theta_low=5;%boundaries for for adaptivce theta
% theta_high=n_adaptive_angles-5;
% spread_matrix=zeros(1,length(angles));
% spread_matrix_adaptive=zeros(2,nsteps,length(angles_adaptive));
% baseline_var=zeros(2,nsteps,length(angles_baseline));
% spread_matrix_fine=zeros(2,nsteps,length(angles_fine));
% thetas=zeros(2,nsteps);%the angle of the lines
data_var=zeros(nsteps,1);
data_hold=zeros(windowsize,npoints);

%data caching
BG_ave_time=5;
framerate=round(mpP.Header.LineRate/mpP.ysize);
nframes_to_cache=round(mpP.Header.LineRate/mpP.ysize);

frame_counter=2;
line_counter=3*stepsize;
lines_per_frame=mpP.ysize;
cached_lines=1:(nframes_to_cache*mpP.ysize);
use_lines=1:windowsize;
data_cache_lum=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+BG_ave_time*framerate)],the_x);
data_cache_lum_uf=(LinescanFramesUnfold(data_cache_lum,BG_ave_time*framerate));
diameter_background_threshold=[];
mpP.Diameter.mean_BG=(mean(data_cache_lum_uf));

data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
size(data_cache_folded)
data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
%

    data_hold=zeros(1,windowsize,length([the_x(1):the_decimate:the_x(2)-1]));
    data_hold(1,:,:)=double(data_cache(1:windowsize,1:the_decimate:end-1));
    %data_hold(2,:,:)=double(data_cache(2:2:windowsize,end:-the_decimate:2));


frame_counter=frame_counter+nframes_to_cache;
data_poop=[data_hold];
% figure(44)
% imagesc(data_hold)
% frame_mean=mean(double(data_cache));
seperabilty_score=100;

%non-interleaved
for k=1:nsteps
    line_counter=line_counter+stepsize;
    frame_line_marker=find(cached_lines==line_counter+1);%find the new place in the frame
    %%%check to see if the data hold will overrun the cache
    n_overrunlines=line_counter-max(cached_lines)+stepsize;%do we run over?
    if(n_overrunlines>=0)%if we over run the data cach, load more
        data_hold(1,1:3*stepsize,:)=(data_hold(1,stepsize+1:end,:));
        % data_hold(1,:,:)=(data_hold(1,:,:));
        % data_hold(2,1:3*stepsize/2,:)=(data_hold(2,stepsize/2+1:end,:));
        if n_overrunlines>0%went past the end of the frame
            data_hold(1,3*stepsize+1:3*stepsize+(stepsize-n_overrunlines),:)=...
                double(data_cache(frame_line_marker:frame_line_marker+(stepsize-n_overrunlines)-1,1:the_decimate:end-1));%put the last lines in
            %data_hold(2,3*stepsize/2+1:3*stepsize/2+(stepsize-n_overrunlines)/2,:)=...
            %double(data_cache(frame_line_marker+1:2:frame_line_marker+(stepsize-n_overrunlines)-1,end:-the_decimate:2));%put the last lines in
            data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
            data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
            cached_lines=cached_lines+lines_per_frame*nframes_to_cache;
            frame_counter=frame_counter+nframes_to_cache;
            frame_line_marker=find(cached_lines-(stepsize-n_overrunlines)==line_counter+1);%find the new place in the frame
            data_hold(1,4*stepsize-n_overrunlines+1:end,:)=...
                double(data_cache(frame_line_marker+1:frame_line_marker+(n_overrunlines),1:the_decimate:end-1));
            %data_hold(2,3*stepsize/2+(stepsize-n_overrunlines)/2+1:end,:)=...
            %double(data_cache(frame_line_marker+1:2:frame_line_marker+(n_overrunlines),end:-the_decimate:2));
            disp(['load frame ' num2str(frame_counter)])
        elseif(n_overrunlines==0)%went to end of frame and no more
            data_hold(1,3*stepsize+1:3*stepsize+(stepsize-n_overrunlines),:)=...
                double(data_cache(frame_line_marker:frame_line_marker+(stepsize-n_overrunlines)-1,1:the_decimate:end-1));%put the last lines in
            %data_hold(2,3*stepsize/2+1:3*stepsize/2+(stepsize-n_overrunlines)/2,:)=...
            %double(data_cache(frame_line_marker+1:2:frame_line_marker+(stepsize-n_overrunlines)-1,end:-the_decimate:2));%put the last lines in
            data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
            data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
            cached_lines=cached_lines+lines_per_frame*nframes_to_cache;
            frame_counter=frame_counter+nframes_to_cache;
            disp(['load frame ' num2str(frame_counter)])
        end
    end
    if(n_overrunlines<0)%keep going using the currently loaded frames
        data_hold(1,:,:)=[squeeze(data_hold(1,stepsize+1:end,:)); (double(data_cache(frame_line_marker:frame_line_marker+stepsize-1,1:the_decimate:end-1)))];
        %   data_hold(2,:,:)=[squeeze(data_hold(2,stepsize/2+1:end,:)); (double(data_cache(frame_line_marker+1:2:frame_line_marker+stepsize,end:-the_decimate:2)))];
    end
    use_lines=use_lines+stepsize;
    the_t(k)=1+(k-1)*stepsize+windowsize/2;
    npoints_decimated=size(data_hold,3);

    %figure out the diameter
    data_hold_mean=mean(double(squeeze(data_hold(1,:,:))));%average all the data in the window in order to estimate diameter
    mpP.Diameter.raw_diameter(k)=calcFWHM(data_hold_mean);
    mpP.Diameter.image_sum(k,:)=data_hold_mean;
end
%end of radon loop
%%
%thetas=thetas-90;

% objective_mag=1/461.3;%measured on 40x olympus dec12 2007 pd
% if (the_objective==1)&&(the_scanmirrors==1)%40x Olympus
%     objective_microns_per_pixel=1/1.5;%(str2num(mpP.Header.Frame_Width));
% elseif (the_objective==1)&&(the_scanmirrors==2)
%     objective_microns_per_pixel=.899/1.5;
% elseif (the_objective==2)&&(the_scanmirrors==1);
%     objective_microns_per_pixel=1.59;%(str2num(mpP.Header.Frame_Width));
% elseif (the_objective==2)&&(the_scanmirrors==2);
%     objective_microns_per_pixel=1.43;%(str2num(mpP.Header.Frame_Width));
% end
objective_microns_per_pixel=user_microns_per_pixel;
if strcmp(mpP.Header.Scan_Mode,'Line Scan')
    Xfactor=objective_microns_per_pixel/(str2num(mpP.Header.Magnification(2:end)));%objective_mag*microns_per_pixel/str2num(mpP.Header.Magnification(2:end));
    Tfactor =mpP.Header.LineRate;
elseif  strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
    Xfactor=objective_microns_per_pixel*((mpP.scanData.scanVelocity)/(4/512));%4volts/512
    Tfactor=mpP.Header.LineRate;
end

mpP.Xfactor=Xfactor;
mpP.Tfactor=Tfactor;
mpP.Diameter.the_decimate=the_decimate;
mpP.Diameter.diameter=the_decimate*Xfactor*mpP.Diameter.raw_diameter;
mpP.num_frames
diameter=mpP.Diameter.diameter;
the_date=date;
mpP.Diameter.the_t=the_t/(mpP.Header.LineRate);
mpP.Diameter.the_Fs=4/window_time_size;
params.Fs=1/(mpP.Diameter.the_t(2)-mpP.Diameter.the_t(1));
try %to load framedata
    data_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [theframes(1) theframes(end)],the_x);
    mpP.Diameter.Image=LinescanFramesUnfoldMPScan(data_folded)';
catch
    mpP.Diameter.Image=zeros(mpP.num_frames*str2num(mpP.Header.Frame_Height),1);
end

try
    for chNum = 3%1 : 4
        chNumStr = num2str(chNum);
        if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
            mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[2 str2num(mpP.Header.Frame_Count)],the_x);
        end
    end
    mpP.ECoG.Fs=1/length(mpP.Ch3)/str2num(mpP.Header.Frame_Count)*((1/mpP.Header.LineRate)/str2num(mpP.Header.Frame_Height));
end

save(['mv_mpP_' fname(1:end-4) '_' the_date],'mpP')


