function [mpP,function_ID]=GetVelocityRadon_MPScan_L4_cc(fname,matfilename,theframes,window_time_size,angle_span, angle_resolution,the_x,the_decimate, max_v, user_microns_per_pixel,channel,isinterleaved);

%filename,matfilename,...
%            [startframe endframe],windowsize,angle_span, angle_resolution,[xst%art(v) xstop(v)],...
%            the_decimate,max_v,user_microns_per_pixel,channel,the_scanmirrors)%funtion for loading *.MPD linescan files and performing the Radon transform on
%them in order to get the RBC velocity
%
%becase MPScan gets line scans in both directions, the frame is split into even
%and odd lines and velocity is extravted independently from each one and then averaged
%INPUTS:
%fname-filename
%matfilename-matalab file containg scan path etc
%theframes - beginning and ending frames
%windowtimesize - size (in sec) of the advancemt of the Radon window
%anglespan - how many degrees (+/-) to look in the adaptive search
%angle_resolution - fine angle resolution
%the_x - [left  right] bounds of which to extract from the frames in the X
%dimension
%the_decimate - factor by which to decimate the line scan in the X dimension
%max_v - time points above this absolute speed are clipped out and interpolated
%between the last and next valid time points
%the_objective - a number designating which objective is used isinterleaved
%user_microns_per_pixel-at 1x for current objective+microscope setup

%OUTPUTS
%thetas - angles of forward and back linescans
%V_hold - velocities of forward and back linescans
%v_out - 0.5*(v_forward+v_back)
%mpP -  a structure with the relevant data
%mpP.Header substruct
%mpP.Ch1 first two frames from channel 1
%mpP.Blood_flow structure with velocity info etc.

%pjd sept 2 2009


%Open stream
function_ID = mfilename('fullpath');
figure(1)
close(1)
f1 = figure(1);
set(f1,'visible','on');
mpfile = actxcontrol('MPfile.Data',[0,0,500,500],f1);
OpenFile = invoke(mpfile,'OpenMPFile',fname);
    

%Read header and take further action based on header information
mpP.Header = mp2mat_readHeader(mpfile,fname);
mpP.num_frames = theframes(end)-theframes(1);%str2double(mpContent.Header.Frame_Count);
mpP.xsize = str2double(mpP.Header.Frame_Width);
mpP.ysize = str2double(mpP.Header.Frame_Height);
mpP.xstart=the_x(1);
mpP.xstop=the_x(end);
frame_height=mpP.ysize;


for chNum = channel%1 : 4  %load 
    chNumStr = num2str(chNum);
    if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
        mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[1 2],the_x);
    end
end



    try
    load(matfilename)
    mpP.scanData=scanData;
    catch    
            mpP.scan_path=[];
        disp('matfile load full of FAIL')
    end
 



first_frames=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, theframes(1):theframes(1)+10,the_x);%get first 10 frames for background subtraction

if strcmp(mpP.Header.Scan_Mode,'Line Scan')
time_per_line=str2num(mpP.Header.Frame_Width)*(5/4)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
isinterleaved=1;
end

if strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
    time_per_line=str2num(mpP.Header.Frame_Width)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
    isinterleaved=0; %arb scans are not interleace 
end
mpP.Header.LineRate=1/time_per_line;
Fs_Blood_Flow=mpP.Header.LineRate;
windowsizea=round(Fs_Blood_Flow*window_time_size);%*.25 %for interlevaed lines

%adapt windowsize to interleaved/not interleaved
if isinterleaved==1
    windowsize=8*round(windowsizea/8);
else
    windowsize=4*round(windowsizea/4);
end
mpP.Blood_flow.windowsize=windowsize;


%%%%basic stuff
nframes=theframes(2)-theframes(1)+1;
stepsize=.25*windowsize;
nlines=nframes*str2num(mpP.Header.Frame_Height);
npoints=max(the_x)-min(the_x)+1;%str2num(mpP.Header.Frame_Width);

nsteps=floor(nlines/stepsize)-3;
angles=(1:180);%parameters for initial search
angles_baseline=1:15:180;
angles_adaptive=[-angle_span:1:angle_span]; %angle range to look over 
angles_fine=-1.75:angle_resolution:1.75; % fine grained search
n_adaptive_angles=length(angles_adaptive);
theta_low=5;%boundaries for for adaptivce theta
theta_high=n_adaptive_angles-5;
spread_matrix=zeros(1,length(angles));
spread_matrix_adaptive=zeros(2,nsteps,length(angles_adaptive));
baseline_var=zeros(2,nsteps,length(angles_baseline));
spread_matrix_fine=zeros(2,nsteps,length(angles_fine));
thetas=zeros(2,nsteps);%the angle of the lines
data_var=zeros(nsteps,1);
data_hold=zeros(windowsize,npoints);

%data caching
BG_ave_time=5;
framerate=round(mpP.Header.LineRate/mpP.ysize);
nframes_to_cache=round(mpP.Header.LineRate/mpP.ysize);

frame_counter=2;
line_counter=3*stepsize;
lines_per_frame=mpP.ysize;
cached_lines=1:(nframes_to_cache*mpP.ysize);
use_lines=1:windowsize;
data_cache_lum=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+BG_ave_time*framerate)],the_x);
data_cache_lum_uf=(LinescanFramesUnfold(data_cache_lum,BG_ave_time*framerate));

mpP.Blood_flow.mean_BG=(mean(data_cache_lum_uf));

data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
size(data_cache_folded)
data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
%
if isinterleaved==1
    data_hold=zeros(2,windowsize/2,length([the_x(1):the_decimate:the_x(2)-1]));
    data_hold(1,:,:)=double(data_cache(1:2:windowsize,1:the_decimate:end-1));
    data_hold(2,:,:)=double(data_cache(2:2:windowsize,end:-the_decimate:2));
else
    data_hold=zeros(1,windowsize,length([the_x(1):the_decimate:the_x(2)-1]));
    data_hold(1,:,:)=double(data_cache(1:windowsize,1:the_decimate:end-1));
end


frame_counter=frame_counter+nframes_to_cache;
data_poop=[data_hold];

seperabilty_score=100;
%%
if isinterleaved==1
    for k=1:nsteps
        line_counter=line_counter+stepsize;
        frame_line_marker=find(cached_lines==line_counter+1);%find the new place in the frame
        %%%check to see if the data hold will overrun the cache
        n_overrunlines=line_counter-max(cached_lines)+stepsize;%do we run over?
        if(n_overrunlines>=0)%if we over run the data cach, load more
            data_hold(1,1:3*stepsize/2,:)=(data_hold(1,stepsize/2+1:end,:));
            data_hold(2,1:3*stepsize/2,:)=(data_hold(2,stepsize/2+1:end,:));
            if n_overrunlines>0%went past the end of the frame
                data_hold(1,3*stepsize/2+1:3*stepsize/2+(stepsize-n_overrunlines)/2,:)=...
                    double(data_cache(frame_line_marker:2:frame_line_marker+(stepsize-n_overrunlines)-2,1:the_decimate:end-1));%put the last lines in
                data_hold(2,3*stepsize/2+1:3*stepsize/2+(stepsize-n_overrunlines)/2,:)=...
                    double(data_cache(frame_line_marker+1:2:frame_line_marker+(stepsize-n_overrunlines)-1,end:-the_decimate:2));%put the last lines in
                data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
                data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
                cached_lines=cached_lines+lines_per_frame*nframes_to_cache;
                frame_counter=frame_counter+nframes_to_cache;
                frame_line_marker=find(cached_lines-(stepsize-n_overrunlines)==line_counter+1);%find the new place in the frame
                data_hold(1,3*stepsize/2+(stepsize-n_overrunlines)/2+1:end,:)=...
                    double(data_cache(frame_line_marker+1:2:frame_line_marker+(n_overrunlines),1:the_decimate:end-1));
                data_hold(2,3*stepsize/2+(stepsize-n_overrunlines)/2+1:end,:)=...
                    double(data_cache(frame_line_marker+1:2:frame_line_marker+(n_overrunlines),end:-the_decimate:2));
                disp(['load frame ' num2str(frame_counter)])
            elseif(n_overrunlines==0)%went to end of frame and no more
                data_hold(1,3*stepsize/2+1:3*stepsize/2+(stepsize-n_overrunlines)/2,:)=...
                    double(data_cache(frame_line_marker:2:frame_line_marker+(stepsize-n_overrunlines)-2,1:the_decimate:end-1));%put the last lines in
                data_hold(2,3*stepsize/2+1:3*stepsize/2+(stepsize-n_overrunlines)/2,:)=...
                    double(data_cache(frame_line_marker+1:2:frame_line_marker+(stepsize-n_overrunlines)-1,end:-the_decimate:2));%put the last lines in
                data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
                data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
                cached_lines=cached_lines+lines_per_frame*nframes_to_cache;
                frame_counter=frame_counter+nframes_to_cache;
                disp(['load frame ' num2str(frame_counter)])
            end
        end
        if(n_overrunlines<0)%keep going using the currently loaded frames
            data_hold(1,:,:)=[squeeze(data_hold(1,stepsize/2+1:end,:)); (double(data_cache(frame_line_marker:2:frame_line_marker+stepsize-1,1:the_decimate:end-1)))];
            data_hold(2,:,:)=[squeeze(data_hold(2,stepsize/2+1:end,:)); (double(data_cache(frame_line_marker+1:2:frame_line_marker+stepsize,end:-the_decimate:2)))];
        end
        use_lines=use_lines+stepsize;
        %take out the local mean of the window
        the_t(k)=1+(k-1)*stepsize+windowsize/2;
        npoints_decimated=size(data_hold,3);
        %  size(mean(data_cache(:,[the_x(2):-the_decimate:the_x(1)])))
        for n=1:npoints_decimated
            data_hold_ms(1,:,n)=data_hold(1,:,n)-mean(mpP.Blood_flow.mean_BG(:,n));
            data_hold_ms(2,:,n)=data_hold(2,:,n)-mean(mpP.Blood_flow.mean_BG(:,end-n));
        end
        data_hold_ms=data_hold_ms-mean(data_hold_ms(:));
        if (k==1 )
            radon_hold(1,:,:)=(radon(squeeze(data_hold_ms(1,:,:)),angles));
            radon_hold(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),angles);
            spread_matrix(1,:)=var(squeeze(radon_hold(1,:,:)));
            spread_matrix(2,:)=var(squeeze(radon_hold(2,:,:)));
            [m(1,:) the_theta(1,:)]=max(spread_matrix(1,k));
            [m(2,:) the_theta(2,:)]=max(spread_matrix(2,k));
            thetas(1,k)=angles(the_theta(1,:));
            thetas(2,k)=angles(the_theta(2,:));
            radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),-thetas(1,k)+angles_adaptive);
            radon_hold_adaptive(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),-thetas(2,k)+angles_adaptive);
            spread_matrix_adaptive(1,k,1)=max(var(squeeze(radon_hold_adaptive(1,:,:))));
            spread_matrix_adaptive(2,k,1)=max(var(squeeze(radon_hold_adaptive(2,:,:))));
            radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)-mean(data_hold_ms(:))),thetas(1,k)+angles_fine);
            radon_hold_fine(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)-mean(data_hold_ms(:))),thetas(2,k)+angles_fine);
            spread_matrix_fine(1,k,:)=squeeze(var(squeeze(radon_hold_fine(1,:,:))));
            spread_matrix_fine(2,k,:)=squeeze(var(squeeze(radon_hold_fine(2,:,:))));
            [m the_theta(1,:)]=max(squeeze(spread_matrix_fine(1,k,:)));
            [m the_theta(2,:)]=max(squeeze(spread_matrix_fine(2,k,:)));
            thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));
            thetas(2,k)=thetas(2,k)+angles_fine(the_theta(2,:));
            baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));
            baseline_var(2,k,:)=var(radon(squeeze(data_hold_ms(2,:,:)),angles_baseline));

            %otherwise, use last angle as starting point
        else
            thetas(1,k)=thetas(1,k-1);
            thetas(2,k)=thetas(2,k-1);
            %if the estimate is poor, look over all angles
            if (seperabilty_score>3)
                radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k-1)+angles_adaptive);
                radon_hold_adaptive(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),thetas(2,k-1)+angles_adaptive);
            else
                radon_hold(1,:,:)=(radon(squeeze(data_hold_ms(1,:,:)),angles));
                radon_hold(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),angles);
                spread_matrix(1,:)=var(squeeze(radon_hold(1,:,:)));
                spread_matrix(2,:)=var(squeeze(radon_hold(2,:,:)));
                [m(1,:) the_theta(1,:)]=max(spread_matrix(1,:));
                [m(2,:) the_theta(2,:)]=max(spread_matrix(2,:));
                thetas(1,k)=angles(the_theta(1,:));
                thetas(2,k)=angles(the_theta(2,:));
                radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),-thetas(1,k)+angles_adaptive);
                radon_hold_adaptive(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),-thetas(2,k)+angles_adaptive);
            end
            spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));
            spread_matrix_adaptive(2,k,:)=var(squeeze(radon_hold_adaptive(2,:,:)));
            [m the_theta(1,:)]=max(spread_matrix_adaptive(1,k,:));
            [m the_theta(2,:)]=max(spread_matrix_adaptive(2,k,:));

            baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%figure out the average radon of the section for calculating S/N
            baseline_var(2,k,:)=var(radon(squeeze(data_hold_ms(2,:,:)),angles_baseline));%figure out the average radon of the section for calculating S/N

            while ((the_theta(1,:)<=theta_low)||(the_theta(1,:)>=theta_high))
                thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
                radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)), thetas(1,k)+angles_adaptive);
                spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));
                [m the_theta(1,:)]=max(squeeze(spread_matrix_adaptive(1,k,:)));
            end
            while ((the_theta(2,:)<=theta_low)||(the_theta(2,:)>=theta_high))
                thetas(2,k)=thetas(2,k)+angles_adaptive(the_theta(2,:));
                radon_hold_adaptive(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)), thetas(2,k)+angles_adaptive);
                spread_matrix_adaptive(2,k,:)=var(squeeze(radon_hold_adaptive(2,:,:)));
                [m the_theta(2,:)]=max(squeeze(spread_matrix_adaptive(2,k,:)));
            end

            baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%
            baseline_var(2,k,:)=var(radon(squeeze(data_hold_ms(2,:,:)),angles_baseline));%
            seperabilty_score=m/mean(squeeze(baseline_var(1,k,:)));
            thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
            radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k)+angles_fine);
            spread_matrix_fine(1,k,:)=var(squeeze(radon_hold_fine(1,:,:)));
            [m the_theta(1,:)]=max(spread_matrix_fine(1,k,:));
            thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));
            thetas(2,k)=thetas(2,k)+angles_adaptive(the_theta(2,:));
            radon_hold_fine(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),thetas(2,k)+angles_fine);
            spread_matrix_fine(2,k,:)=var(squeeze(radon_hold_fine(2,:,:)));
            [m the_theta(2,:)]=max(spread_matrix_fine(2,k,:));
            thetas(2,k)=thetas(2,k)+angles_fine(the_theta(2,:));
        end
    end
    %%
else %non-interleaved
    for k=1:nsteps
        line_counter=line_counter+stepsize;
        frame_line_marker=find(cached_lines==line_counter+1);%find the new place in the frame
        %%%check to see if the data hold will overrun the cache
        n_overrunlines=line_counter-max(cached_lines)+stepsize;%do we run over?
        if(n_overrunlines>=0)%if we over run the data cach, load more
            data_hold(1,1:3*stepsize,:)=(data_hold(1,stepsize+1:end,:));
            if n_overrunlines>0%went past the end of the frame
                data_hold(1,3*stepsize+1:3*stepsize+(stepsize-n_overrunlines),:)=...
                    double(data_cache(frame_line_marker:frame_line_marker+(stepsize-n_overrunlines)-1,1:the_decimate:end-1));%put the last lines in
                data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
                data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
                cached_lines=cached_lines+lines_per_frame*nframes_to_cache;
                frame_counter=frame_counter+nframes_to_cache;
                frame_line_marker=find(cached_lines-(stepsize-n_overrunlines)==line_counter+1);%find the new place in the frame
                data_hold(1,4*stepsize-n_overrunlines+1:end,:)=...
                    double(data_cache(frame_line_marker+1:frame_line_marker+(n_overrunlines),1:the_decimate:end-1));
                %data_hold(2,3*stepsize/2+(stepsize-n_overrunlines)/2+1:end,:)=...
                %double(data_cache(frame_line_marker+1:2:frame_line_marker+(n_overrunlines),end:-the_decimate:2));
                disp(['load frame ' num2str(frame_counter)])
            elseif(n_overrunlines==0)%went to end of frame and no more
                data_hold(1,3*stepsize+1:3*stepsize+(stepsize-n_overrunlines),:)=...
                    double(data_cache(frame_line_marker:frame_line_marker+(stepsize-n_overrunlines)-1,1:the_decimate:end-1));%put the last lines in
                %data_hold(2,3*stepsize/2+1:3*stepsize/2+(stepsize-n_overrunlines)/2,:)=...
                %double(data_cache(frame_line_marker+1:2:frame_line_marker+(stepsize-n_overrunlines)-1,end:-the_decimate:2));%put the last lines in
                data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache)],the_x);
                data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
                cached_lines=cached_lines+lines_per_frame*nframes_to_cache;
                frame_counter=frame_counter+nframes_to_cache;
                disp(['load frame ' num2str(frame_counter)])
            end
        end
        if(n_overrunlines<0)%keep going using the currently loaded frames
            data_hold(1,:,:)=[squeeze(data_hold(1,stepsize+1:end,:)); (double(data_cache(frame_line_marker:frame_line_marker+stepsize-1,1:the_decimate:end-1)))];
        end
        use_lines=use_lines+stepsize;
        %take out the local mean of the window
        the_t(k)=1+(k-1)*stepsize+windowsize/2;
        npoints_decimated=size(data_hold,3);
        for n=1:npoints_decimated
            data_hold_ms(1,:,n)=data_hold(1,:,n)-mean(mpP.Blood_flow.mean_BG(:,n));
        end
        data_hold_ms=data_hold_ms-mean(data_hold_ms(:));
        if (k==1 )
            radon_hold(1,:,:)=(radon(squeeze(data_hold_ms(1,:,:)),angles));
            spread_matrix(1,:)=var(squeeze(radon_hold(1,:,:)));
            [m(1,:) the_theta(1,:)]=max(spread_matrix(1,k));
            thetas(1,k)=angles(the_theta(1,:));
            radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),-thetas(1,k)+angles_adaptive);
            spread_matrix_adaptive(1,k,1)=max(var(squeeze(radon_hold_adaptive(1,:,:))));
            radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)-mean(data_hold_ms(:))),thetas(1,k)+angles_fine);
            spread_matrix_fine(1,k,:)=squeeze(var(squeeze(radon_hold_fine(1,:,:))));
            [m the_theta(1,:)]=max(squeeze(spread_matrix_fine(1,k,:)));
            thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));
            baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));

            %otherwise, use last angle as starting point
        else
            thetas(1,k)=thetas(1,k-1);
            if (seperabilty_score>3)
                radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k-1)+angles_adaptive);
            else
                radon_hold(1,:,:)=(radon(squeeze(data_hold_ms(1,:,:)),angles));
                spread_matrix(1,:)=var(squeeze(radon_hold(1,:,:)));
                [m(1,:) the_theta(1,:)]=max(spread_matrix(1,:));
                thetas(1,k)=angles(the_theta(1,:));
                radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),-thetas(1,k)+angles_adaptive);
            end
            spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));
            [m the_theta(1,:)]=max(spread_matrix_adaptive(1,k,:));

            baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%figure out the average radon of the section for calculating S/N


            while ((the_theta(1,:)<=theta_low)||(the_theta(1,:)>=theta_high))
                thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
                radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)), thetas(1,k)+angles_adaptive);
                spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));
                [m the_theta(1,:)]=max(squeeze(spread_matrix_adaptive(1,k,:)));
            end
    
            baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%
            seperabilty_score=m/mean(squeeze(baseline_var(1,k,:)));
            thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
            radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k)+angles_fine);
            spread_matrix_fine(1,k,:)=var(squeeze(radon_hold_fine(1,:,:)));
            [m the_theta(1,:)]=max(spread_matrix_fine(1,k,:));
            thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));
          
        end
    end
end
%end of radon loop

%%
%thetas=thetas-90;
if((length(find(thetas<0))>=1)&&(length(find(thetas>0))>=1))
    disp('sign flip!')
end

size(spread_matrix_adaptive);
mpP.Blood_flow.thetas=thetas;

mpP.Blood_flow.unscaled_velocity=(cotd(thetas-90));
mpP.Blood_flow.spread_matrix_adaptive=spread_matrix_adaptive;
mpP.Blood_flow.baseline_var=baseline_var;

objective_microns_per_pixel=user_microns_per_pixel;%
if strcmp(mpP.Header.Scan_Mode,'Line Scan')
    Xfactor=objective_microns_per_pixel/(str2num(mpP.Header.Magnification(2:end)));%objective_mag*microns_per_pixel/str2num(mpP.Header.Magnification(2:end));
    Tfactor =mpP.Header.LineRate;
elseif  strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
    Xfactor=objective_microns_per_pixel*((mpP.scanData.scanVelocity)/(4/512));%4 volts/512 pixels
    Tfactor=mpP.Header.LineRate;
end

mpP.Xfactor=Xfactor;
mpP.Tfactor=Tfactor;
mpP.Blood_flow.the_decimate=the_decimate;
if isinterleaved==1 %need to divide by 2 because of splitting due to inerleaving 
    mpP.Blood_flow.v(1,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(1,:)-90))/2;
    mpP.Blood_flow.v(2,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(2,:)-90))/2;
else     
    mpP.Blood_flow.v(1,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(1,:)-90));
    mpP.Blood_flow.v(2,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(1,:)-90));
end

mpP.Blood_flow.velocity2(1,:)=VelocityCleanUp(mpP.Blood_flow.v(1,:),max_v);

if isinterleaved==1
    mpP.Blood_flow.velocity2(2,:)=VelocityCleanUp(mpP.Blood_flow.v(2,:),max_v);
else
    mpP.Blood_flow.velocity2(2,:)=-(mpP.Blood_flow.velocity2(1,:));
end

v_hold=mpP.Blood_flow.velocity2;
v_out=0.5*(v_hold(1,:)-v_hold(2,:));%take the average of the two directions
mpP.Blood_flow.v_out=v_out;
mpP.Blood_flow.the_t=the_t/(mpP.Header.LineRate);
mpP.Blood_flow.the_Fs=4/window_time_size;
mpP.num_frames

the_date=date;
[fixed_v]=VelocityCleanUpSTDOutliers(mpP.Blood_flow.v_out, 3);%
mpP.Blood_flow.fixed_v=fixed_v;
params.Fs=1/(mpP.Blood_flow.the_t(2)-mpP.Blood_flow.the_t(1));
[Shr,thr,fhr]=mtdspecgramc(diff(mpP.Blood_flow.fixed_v),[2 .5],[pi/2],params);

mpP.Blood_flow.velocity_specgram=Shr;
mpP.Blood_flow.velocity_specgram_t=thr;
try %to load raw data
    data_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [theframes(1) theframes(end)],the_x);
    mpP.Blood_flow.Image=LinescanFramesUnfoldMPScan(data_folded)';
catch
    mpP.Blood_flow.Image=zeros(mpP.num_frames*str2num(mpP.Header.Frame_Height),1);
end



seperability=max(squeeze(mean(mpP.Blood_flow.spread_matrix_adaptive,1))')./mean(squeeze(mean(mpP.Blood_flow.baseline_var,1))');%
mpP.Blood_flow.seperability=seperability;
sep_threshold=3;
low_sep=find(seperability<sep_threshold);
try
    for chNum = 3%1 : 4
        chNumStr = num2str(chNum);
        if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
            mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[2 str2num(mpP.Header.Frame_Count)],the_x);
        end
    end

    mpP.ECoG.Fs=1/length(mpP.Ch3)/str2num(mpP.Header.Frame_Count)*((1/mpP.Header.LineRate)/str2num(mpP.Header.Frame_Height));
end

save(['mv_mpP_' fname(1:end-4) '_' the_date],'mpP')

%% dependent functions
function [fixed_v]=VelocityCleanUpSTDOutliers(v, thresh)
%finds points outside a threshold velocity and interpolated between last
% and next good points, unless at the begining or end, then puts in mean
% velocities
%pd 2-21-08
upper_bound=thresh*std(v)+mean(v);
lower_bound=-thresh*std(v)+mean(v);
badpoints=union(find(v>upper_bound),find(v<lower_bound));
noutliers=length(badpoints);
fixed_v=v;
while noutliers>0
%badpoints=find(abs(v)>thresh);
goodpoints=union(setdiff(badpoints+1,badpoints),setdiff(badpoints-1,badpoints));
for p=1:length(badpoints)
    lastgood=max(find(goodpoints<badpoints(p)));
    nextgood=min(find(goodpoints>badpoints(p)));
    if ((goodpoints(1)>0)&&(goodpoints(end)<=length(fixed_v)))
        if((numel(lastgood)>0)&&(isempty(nextgood)==0))
        fixed_v(badpoints(p))=.5*fixed_v(goodpoints(lastgood))+.5*fixed_v(goodpoints(nextgood));
        elseif((numel(lastgood)>0))
        fixed_v(badpoints(p))=fixed_v(goodpoints(nextgood)); 
        else
        fixed_v(badpoints(p))=fixed_v(goodpoints(lastgood)); 
        end
    elseif (goodpoints(1)==0)
     fixed_v(badpoints(p))=mean(fixed_v);%(goodpoints(nextgood));
     elseif (goodpoints(end)==length(fixed_v)+1)
     fixed_v(badpoints(p))=mean(fixed_v);%fixed_v(goodpoints(lastgood)); 
    elseif numel(goodpoints)==0
         fixed_v(badpoints(p))=-fixed_v;
    end
    
end
upper_bound=thresh*std(fixed_v)+mean(fixed_v);
lower_bound=-thresh*std(fixed_v)+mean(fixed_v);
badpoints=union(find(fixed_v>upper_bound),find(fixed_v<lower_bound));
noutliers=length(badpoints);

end
function Header = mp2mat_readHeader(mpfile,fname)
%Read all the header information, but do nothing with it
Header.Scan_Mode = invoke(mpfile,'ReadParameter','Scan Mode');
Header.Frame_Width = invoke(mpfile,'ReadParameter','Frame Width');
Header.Frame_Height = invoke(mpfile,'ReadParameter','Frame Height');
Header.Frame_Count = invoke(mpfile,'ReadParameter','Frame Count');
Header.X_Position = invoke(mpfile,'ReadParameter','X Position');
Header.Y_Position = invoke(mpfile,'ReadParameter','Y Position');
Header.Z_Position = invoke(mpfile,'ReadParameter','Z Position');
Header.Stack_Count = invoke(mpfile,'ReadParameter','Stack Count');
Header.Z_Interval = invoke(mpfile,'ReadParameter','z- Interval');
Header.Averaging_Count = invoke(mpfile,'ReadParameter','Averaging Count');
Header.Repeat_Count = invoke(mpfile,'ReadParameter','Repeat Count');
Header.Magnification = invoke(mpfile,'ReadParameter','Magnification');
Header.Rotation = invoke(mpfile,'ReadParameter','Rotation');
Header.X_Frame_Offset = invoke(mpfile,'ReadParameter','X Frame Offset');
Header.Y_Frame_Offset = invoke(mpfile,'ReadParameter','Y Frame Offset');
Header.Channel_Name1 = invoke(mpfile,'ReadParameter','Channel Name (1)');
Header.Channel_Name2 = invoke(mpfile,'ReadParameter','Channel Name (2)');
Header.Channel_Name3 = invoke(mpfile,'ReadParameter','Channel Name (3)');
Header.Channel_Name4 = invoke(mpfile,'ReadParameter','Channel Name (4)');
Header.Enabled1 = invoke(mpfile,'ReadParameter','Enabled (1)');
Header.Enabled2 = invoke(mpfile,'ReadParameter','Enabled (2)');
Header.Enabled3 = invoke(mpfile,'ReadParameter','Enabled (3)');
Header.Enabled4 = invoke(mpfile,'ReadParameter','Enabled (4)');
Header.Input_Range1 = invoke(mpfile,'ReadParameter','Input Range (1)');
Header.Input_Range2 = invoke(mpfile,'ReadParameter','Input Range (2)');
Header.Input_Range3 = invoke(mpfile,'ReadParameter','Input Range (3)');
Header.Input_Range4 = invoke(mpfile,'ReadParameter','Input Range (4)');
Header.Channel_Unit3 = invoke(mpfile,'ReadParameter','Channel Unit (3)');
Header.Channel_Unit4 = invoke(mpfile,'ReadParameter','Channel Unit (4)');
Header.Channel_Prefix3 = invoke(mpfile,'ReadParameter','Channel Prefix (3)');
Header.Channel_Prefix4 = invoke(mpfile,'ReadParameter','Channel Prefix (4)');
Header.Conversion_Factor3 = invoke(mpfile,'ReadParameter','Conversion Factor (3)');
Header.Conversion_Factor4 = invoke(mpfile,'ReadParameter','Conversion Factor (4)');
Header.Offset3 = invoke(mpfile,'ReadParameter','Offset (3)');
Header.Offset4 = invoke(mpfile,'ReadParameter','Offset (4)');
Header.Data_Point_Per_Frame3 = invoke(mpfile,'ReadParameter','Data Point Per Frame (3)');
Header.Data_Point_Per_Frame4 = invoke(mpfile,'ReadParameter','Data Point Per Frame (4)');
Header.Pixel_Clock= invoke(mpfile,'ReadParameter','Pixel Clock');
Header.Frame_Rate=invoke(mpfile,'ReadParameter','Frame Rate');
Header.Filename = fname;
Header.Comments = invoke(mpfile,'ReadParameter','Comments');
function chData = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpContent, theframes,the_x)


%read first data frame and figure out if channel contains analog or image data
tempframe = mpfile.ReadFrameData(chNum,1);
xsize=the_x(2)-the_x(1)+1;
myframes=theframes(1):theframes(2);
if numel(tempframe) == mpContent.xsize * mpContent.ysize
   %got image
    
   chData = zeros(mpContent.ysize,xsize,max(theframes)-min(theframes),'uint16');
    temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
    chData(:,:,1)=temp_frame_reshaped(:,the_x(1):the_x(2));
   for nf = 1:length(myframes)
       tempframe = mpfile.ReadFrameData(chNum,myframes(nf));
        temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
         chData(:,:,nf)=temp_frame_reshaped(:,the_x(1):the_x(2));

   end
else
   %got analog data
   np = length(tempframe);
   chData = zeros( np *  mpContent.num_frames,1,'int16');
   idx = 1 : np;
   chData(idx) = tempframe;
   for nf = 1 :theframes(2)
       tempframe = mpfile.ReadFrameData(chNum,nf)';
       idx = idx + np;
       chData(idx) = tempframe;
   end
end
function [frames_hold]=LinescanFramesUnfold(passedframes,nframes);
%changes mpscan linescan data from frames to a  "scroll"
frame_height=size(passedframes,1);
frame_width=size(passedframes,2);
frames_hold=zeros(frame_height*nframes,frame_width,'uint16');
for k=1:nframes
    frames_hold(((k-1)*frame_height+1):((k)*frame_height),:)=squeeze(passedframes(:,:,k));
end
function [fixed_v]=VelocityCleanUp(v, thresh)
%finds points faster than a threshold velocity and interpolated between last
% and next good points
%pd 11-30-07

badpoints=find(abs(v)>thresh);
goodpoints=union(setdiff(badpoints+1,badpoints),setdiff(badpoints-1,badpoints));
fixed_v=v;
for p=1:length(badpoints)
    lastgood=max(find(goodpoints<badpoints(p)));
    nextgood=min(find(goodpoints>badpoints(p)));
    if ((goodpoints(1)>0)&&(goodpoints(end)<=length(v)))

        if((numel(lastgood)>0)&&(isempty(nextgood)==0))
        fixed_v(badpoints(p))=.5*v(goodpoints(lastgood))+.5*v(goodpoints(nextgood));
        elseif((numel(lastgood)>0))
        fixed_v(badpoints(p))=v(goodpoints(nextgood)); 
        else
        fixed_v(badpoints(p))=v(goodpoints(lastgood)); 
        end
    elseif (goodpoints(1)==0)
     fixed_v(badpoints(p))=v(goodpoints(nextgood));
     elseif (goodpoints(end)==length(v)+1)
     fixed_v(badpoints(p))=v(goodpoints(lastgood)); 
    elseif numel(goodpoints)==0
         fixed_v(badpoints(p))=-v;
    end
    
end
function [frames_hold]=LinescanFramesUnfoldMPScan(passedframes);
%changes mpscan linescan data from frames to a  "scroll"
frame_height=size(passedframes,1);
frame_width=size(passedframes,2);
nframes=size(passedframes,3);
frames_hold=zeros(frame_height*nframes,frame_width,'uint16');
for k=1:nframes
    frames_hold(((k-1)*frame_height+1):((k)*frame_height),:)=squeeze(passedframes(:,:,k));
end






