function [mv_mpP]=MPDMultiVessel_cc(sampling_freq,channel);
%opens MPD file, gets xdimension to scan over from user input, makes window size
%sampling_freq - (in Hz)
%channel - pmt channel to use, usually 1
%structures are mv_mpP, with mpP substructs
%pjd 4/11
my_crunched_folder='F:\data\crunched\'
matfilename=[];%contains arbitraryscanpath data
line_boundaries=[]
[filename,pathname]=uigetfile('*.MPD','pick a linescan file')%loads file
try
    [matfilename,matpathname]=uigetfile('*.mat','pick a .mat data file')%loads file
catch
    matfilename='';
end
 
cd(pathname)
%display the file
[xstart, xstop,frames_hold,nframes,vessel_ID,depths,scan_type]=MPD_Display_Frames_MultiVessel(filename,matfilename,channel);
nvessels=length(xstart);%how many vessels?
startframe=input('start frame: ');
endframe=input('end frame: ');
the_decimate=1;%input('decimation factor: '); only for oversampled data
user_microns_per_pixel=input('microns per pixel @ 1x');
the_objective=input('objective: [1]-40x Olympus: [2]-20x, 0.5NA: [3]-20x,0.95NA: [4]-10x: [5]-4x:');
animal=input('animal?','s');

max_v=10000;%input('maximum velocity, in micrometers/sec: ')
windowsize=1/(sampling_freq);
angle_span=15;%number of degrees (+/-) around previous angle to look
angle_resolution=.1;%how accurate to determine the angle
the_scanmirrors=input('which mirrors? 1)6210(fast) 2)6215(slow)')
channel=1;
for v=1:nvessels
    thisfile=mfilename('fullpath');
    mpP.code=GetFunctionCode(thisfile);
    if (scan_type(v)==1)%get velocity
        [mpP,function_ID]=GetVelocityRadon_MPScan_L4_cc(filename,matfilename,...
            [startframe endframe],windowsize,angle_span, angle_resolution,[xstart(v) xstop(v)],...
            the_decimate,max_v,user_microns_per_pixel,channel,the_scanmirrors);%
        try
        mpP.radon_code=GetFunctionCode(function_ID);
        catch
        disp('code read fail!')  
        end
    end
    if (scan_type(v)==0)%get diameter
        [mpP]=GetDiameter_MPScan_L4_cc(filename,matfilename,[startframe endframe],windowsize,...
            [xstart(v) xstop(v)],the_decimate,user_microns_per_pixel,channel);

        try
        mpP.diameter_code=GetFunctionCode(function_ID);
        catch
        disp('code read fail!')  
        end
        try
            figure(556)
            subplot(212)
            plot(mpP.Diameter.the_t,mpP.Diameter.raw_diameter(1,:))
            subplot(211)
            imagesc(mpP.Diameter.Image{1})
        end
    end


    if the_scanmirrors==1
        mpP.the_scan_mirrors=6210;
    elseif the_scanmirrors==2
        mpP.the_scan_mirrors=6215;
    end
    try
        mpP.ECoG.raw=mpP.Ch3;
    end
    mpP.animal=animal;

    try
        mpP.vessel=vessel_ID{v};
    end
    mpP.clock=clock;
    mpP.the_objective=the_objective;
    mpP.stimtime=NaN;
    mpP.depth=depths(v);
    try
        mpP.scan_type=scan_type(v);
    end
    if v>1
        [mv_mpP,mpP]=MatchStructs(mv_mpP,mpP)
        mv_mpP=orderfields(mv_mpP);
        mpP=orderfields(mpP);
    end
    mv_mpP(v)=mpP;%put the substruct in the structure

end
ntrials=1;%input
the_date=date;

save_filename=[animal '_' vessel_ID{1} '_mv_mpP_' filename(1:end-4) '_' the_date];

try
    save(save_filename,'mv_mpP')%save processed file to current directory
    save([my_crunched_folder save_filename],'mv_mpP')%save to crunched directory
catch
    disp('crunched folder save failed!')
end
try
    plot_mv_mpP_cc(mv_mpP,109,'all');
catch
    disp('plot failed')
end
end

%subfunctions
function [the_code]=GetFunctionCode(filename)
%this function the file and returns it as a string
a = fopen([ filename '.m']);
the_code_numbers=fread(a);
the_code = char(the_code_numbers);
end

function [s1,s2]=MatchStructs(s1,s2)
fields1=fieldnames(s1);
fields2=fieldnames(s2);
mpP=s2;
for n=1:length(fields1)
    if (strcmp(fields1{n},fields2)==0)
        for k=1:length(s2)
            s2(k).(fields1{n})=[];%=setfield(s2(k),fields1{n},[]);%put an empty field in
        end
    end
end

for n=1:length(fields2)
    if (strcmp(fields2{n},fields1)==0)
        for k=1:length(s1)
             s1(k).(fields2{n})=[]%s1(k)=setfield(s1(k),fields2{n},[]);%put an empty field in
        end
    end
end
end
function [xstart, xstop,frames_hold,nframes,vessel_ID,depths,scan_type]=MPD_Display_Frames_MultiVessel(fname,mfname,channel)
%this function displays the first frames of the MPD linescan file and gets the
%user input to determine the x range to use for the radon transform
figure(1) 
close(1)
f1 = figure(1);
set(f1,'visible','on');
mpfile = actxcontrol('MPfile.Data',[0,0,500,500],f1);
OpenFile = invoke(mpfile,'OpenMPFile',fname);

%Read header and take further action based on header information
mpP.Header = mp2mat_readHeader(mpfile,fname);
nframes=mpP.Header.Frame_Count
mpP.xsize = str2double(mpP.Header.Frame_Width);
mpP.ysize = str2double(mpP.Header.Frame_Height);
frame_height=mpP.ysize;

first_frames=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [2 2],[1 max(mpP.xsize)] );%get first 3 frames
size(first_frames);
[frames_hold]=LinescanFramesUnfold(first_frames,size(first_frames,3));
size(first_frames);

figure(2)
colormap gray
%plot the data
subplot(211)
hold off
imagesc(double(frames_hold))
hold on
%axis image 
axis off
title(fname)
%plot the linescan trajectory
if length(mfname)>0 %plot the 
try
load(mfname)
line_boundaries=find(abs(diff(scanData.pathObjSubNum)))
scan_velocity=scanData.scanVelocity;
for ss=1:length(line_boundaries)
    plot(line_boundaries(ss)*[1 1],[0 1000],'r')
end
subplot(212)
hold off
imagesc(scanData.im)
axis image
hold on
pathImCoords(:,1) = scanData.path(:,1) * (size(scanData.im,2)-1)/(abs(diff(scanData.axisLimCol))) + 1 - (size(scanData.im,2)-1)/(abs(diff(scanData.axisLimCol)))*min(scanData.axisLimCol);
pathImCoords(:,2) = scanData.path(:,2) * (size(scanData.im,1)-1)/(abs(diff(scanData.axisLimRow))) + 1 - (size(scanData.im,1)-1)/(abs(diff(scanData.axisLimRow)))*min(scanData.axisLimRow);
plot(pathImCoords(:,1),pathImCoords(:,2),'r')

end
end
nvessels=input('How many vessels?')
scan_type=-ones(nvessels,1);
vessel_ID=cell(nvessels,1);
depths=-ones(nvessels,1);

for vessel=1:nvessels
disp(['select vessel #' num2str(vessel)]) 
[x,y]=ginput(2);
xstart(vessel)=round(min(x))
xstop(vessel)=round(max(x))
depths(vessel)=input('depth?')
vessel_ID{vessel}=input('vessel ID?','s')
scan_type(vessel)=input('Scan type? 0=diameter 1=velocity')
end
end
function Header = mp2mat_readHeader(mpfile,fname)
%Read all the header information, but do nothing with it
Header.Scan_Mode = invoke(mpfile,'ReadParameter','Scan Mode');
Header.Frame_Width = invoke(mpfile,'ReadParameter','Frame Width');
Header.Frame_Height = invoke(mpfile,'ReadParameter','Frame Height');
Header.Frame_Count = invoke(mpfile,'ReadParameter','Frame Count');
Header.X_Position = invoke(mpfile,'ReadParameter','X Position');
Header.Y_Position = invoke(mpfile,'ReadParameter','Y Position');
Header.Z_Position = invoke(mpfile,'ReadParameter','Z Position');
Header.Stack_Count = invoke(mpfile,'ReadParameter','Stack Count');
Header.Z_Interval = invoke(mpfile,'ReadParameter','z- Interval');
Header.Averaging_Count = invoke(mpfile,'ReadParameter','Averaging Count');
Header.Repeat_Count = invoke(mpfile,'ReadParameter','Repeat Count');
Header.Magnification = invoke(mpfile,'ReadParameter','Magnification');
Header.Rotation = invoke(mpfile,'ReadParameter','Rotation');
Header.X_Frame_Offset = invoke(mpfile,'ReadParameter','X Frame Offset');
Header.Y_Frame_Offset = invoke(mpfile,'ReadParameter','Y Frame Offset');
Header.Channel_Name1 = invoke(mpfile,'ReadParameter','Channel Name (1)');
Header.Channel_Name2 = invoke(mpfile,'ReadParameter','Channel Name (2)');
Header.Channel_Name3 = invoke(mpfile,'ReadParameter','Channel Name (3)');
Header.Channel_Name4 = invoke(mpfile,'ReadParameter','Channel Name (4)');
Header.Enabled1 = invoke(mpfile,'ReadParameter','Enabled (1)');
Header.Enabled2 = invoke(mpfile,'ReadParameter','Enabled (2)');
Header.Enabled3 = invoke(mpfile,'ReadParameter','Enabled (3)');
Header.Enabled4 = invoke(mpfile,'ReadParameter','Enabled (4)');
Header.Input_Range1 = invoke(mpfile,'ReadParameter','Input Range (1)');
Header.Input_Range2 = invoke(mpfile,'ReadParameter','Input Range (2)');
Header.Input_Range3 = invoke(mpfile,'ReadParameter','Input Range (3)');
Header.Input_Range4 = invoke(mpfile,'ReadParameter','Input Range (4)');
Header.Channel_Unit3 = invoke(mpfile,'ReadParameter','Channel Unit (3)');
Header.Channel_Unit4 = invoke(mpfile,'ReadParameter','Channel Unit (4)');
Header.Channel_Prefix3 = invoke(mpfile,'ReadParameter','Channel Prefix (3)');
Header.Channel_Prefix4 = invoke(mpfile,'ReadParameter','Channel Prefix (4)');
Header.Conversion_Factor3 = invoke(mpfile,'ReadParameter','Conversion Factor (3)');
Header.Conversion_Factor4 = invoke(mpfile,'ReadParameter','Conversion Factor (4)');
Header.Offset3 = invoke(mpfile,'ReadParameter','Offset (3)');
Header.Offset4 = invoke(mpfile,'ReadParameter','Offset (4)');
Header.Data_Point_Per_Frame3 = invoke(mpfile,'ReadParameter','Data Point Per Frame (3)');
Header.Data_Point_Per_Frame4 = invoke(mpfile,'ReadParameter','Data Point Per Frame (4)');
Header.Pixel_Clock= invoke(mpfile,'ReadParameter','Pixel Clock');
Header.Frame_Rate=invoke(mpfile,'ReadParameter','Frame Rate');
Header.Filename = fname;
Header.Comments = invoke(mpfile,'ReadParameter','Comments');
end
function chData = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpContent, theframes,the_x)


%read first data frame and figure out if channel contains analog or image data
tempframe = mpfile.ReadFrameData(chNum,1);
xsize=the_x(2)-the_x(1)+1;
myframes=theframes(1):theframes(2);
if numel(tempframe) == mpContent.xsize * mpContent.ysize
   %got image
    
   chData = zeros(mpContent.ysize,xsize,max(theframes)-min(theframes),'uint16');
    temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
    chData(:,:,1)=temp_frame_reshaped(:,the_x(1):the_x(2));
   for nf = 1:length(myframes)
       tempframe = mpfile.ReadFrameData(chNum,myframes(nf));
        temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
         chData(:,:,nf)=temp_frame_reshaped(:,the_x(1):the_x(2));

   end
else
   %got analog data
   np = length(tempframe);
   chData = zeros( np *  mpContent.num_frames,1,'int16');
   idx = 1 : np;
   chData(idx) = tempframe;
   for nf = 1 :theframes(2)
       tempframe = mpfile.ReadFrameData(chNum,nf)';
       idx = idx + np;
       chData(idx) = tempframe;
   end
end

end
function [mpP, fixed_v_noHR]=HeartRateRemove(mpP)
%removes high freqency (>8hz) noise with 2nd order butterworth filter
f_high=5;
fs=mpP.Blood_flow.the_Fs;
[hr_B,hr_A]=butter(2,2*[f_high]/fs,'low');
for k=1:min(size(mpP.Blood_flow.fixed_v))
    fixed_v_noHR(k,:)=filter(hr_B,hr_A,mpP.Blood_flow.fixed_v(k,:));%filter the velocity signal
end
mpP.Blood_flow.fixed_v_noHR=fixed_v_noHR;
end
function [frames_hold]=LinescanFramesUnfold(passedframes,nframes);
%changes mpscan linescan data from frames to a  "scroll"
frame_height=size(passedframes,1);
frame_width=size(passedframes,2);
frames_hold=zeros(frame_height*nframes,frame_width,'uint16');
for k=1:nframes
    frames_hold(((k-1)*frame_height+1):((k)*frame_height),:)=squeeze(passedframes(:,:,k));
end
end



